@@ -1,7 +1,5 @@
 /* Miscellaneous generic support functions for GNU Make.
-Copyright (C) 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
-1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Free
-Software Foundation, Inc.
+Copyright (C) 1988-2013 Free Software Foundation, Inc.
 This file is part of GNU Make.
 
 GNU Make is free software; you can redistribute it and/or modify it under the
@@ -16,40 +14,13 @@ A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 You should have received a copy of the GNU General Public License along with
 this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "make.h"
+#include "makeint.h"
 #include "dep.h"
 #include "debug.h"
 
-/* Variadic functions.  We go through contortions to allow proper function
-   prototypes for both ANSI and pre-ANSI C compilers, and also for those
-   which support stdarg.h vs. varargs.h, and finally those which have
-   vfprintf(), etc. and those who have _doprnt... or nothing.
+/* GNU make no longer supports pre-ANSI89 environments.  */
 
-   This fancy stuff all came from GNU fileutils, except for the VA_PRINTF and
-   VA_END macros used here since we have multiple print functions.  */
-
-#if USE_VARIADIC
-# if HAVE_STDARG_H
-#  include <stdarg.h>
-#  define VA_START(args, lastarg) va_start(args, lastarg)
-# else
-#  include <varargs.h>
-#  define VA_START(args, lastarg) va_start(args)
-# endif
-# if HAVE_VPRINTF
-#  define VA_PRINTF(fp, lastarg, args) vfprintf((fp), (lastarg), (args))
-# else
-#  define VA_PRINTF(fp, lastarg, args) _doprnt((lastarg), (args), (fp))
-# endif
-# define VA_END(args) va_end(args)
-#else
-/* We can't use any variadic interface! */
-# define va_alist a1, a2, a3, a4, a5, a6, a7, a8
-# define va_dcl char *a1, *a2, *a3, *a4, *a5, *a6, *a7, *a8;
-# define VA_START(args, lastarg)
-# define VA_PRINTF(fp, lastarg, args) fprintf((fp), (lastarg), va_alist)
-# define VA_END(args)
-#endif
+#include <stdarg.h>
 
 
 /* Compare strings *S1 and *S2.
@@ -89,62 +60,64 @@ collapse_continuations (char *line)
   while (*in != '\0')
     {
       /* BS_WRITE gets the number of quoted backslashes at
-	 the end just before IN, and BACKSLASH gets nonzero
-	 if the next character is quoted.  */
+         the end just before IN, and BACKSLASH gets nonzero
+         if the next character is quoted.  */
       backslash = 0;
       bs_write = 0;
       for (p = in - 1; p >= line && *p == '\\'; --p)
-	{
-	  if (backslash)
-	    ++bs_write;
-	  backslash = !backslash;
+        {
+          if (backslash)
+            ++bs_write;
+          backslash = !backslash;
 
-	  /* It should be impossible to go back this far without exiting,
-	     but if we do, we can't get the right answer.  */
-	  if (in == out - 1)
-	    abort ();
-	}
+          /* It should be impossible to go back this far without exiting,
+             but if we do, we can't get the right answer.  */
+          if (in == out - 1)
+            abort ();
+        }
 
       /* Output the appropriate number of backslashes.  */
       while (bs_write-- > 0)
-	*out++ = '\\';
+        *out++ = '\\';
 
       /* Skip the newline.  */
       ++in;
 
-      /* If the newline is escaped, discard following whitespace leaving just
-	 one space.  POSIX requires that each backslash/newline/following
-	 whitespace sequence be reduced to a single space.  */
       if (backslash)
-	{
-	  in = next_token (in);
-          /* Removing this loop will fix Savannah bug #16670: do we want to? */
-	  while (out > line && isblank ((unsigned char)out[-1]))
-	    --out;
-	  *out++ = ' ';
-	}
+        {
+          /* Backslash/newline handling:
+             In traditional GNU make all trailing whitespace, consecutive
+             backslash/newlines, and any leading whitespace on the next line
+             is reduced to a single space.
+             In POSIX, each backslash/newline and is replaced by a space.  */
+          in = next_token (in);
+          if (! posix_pedantic)
+            while (out > line && isblank ((unsigned char)out[-1]))
+              --out;
+          *out++ = ' ';
+        }
       else
-	/* If the newline isn't quoted, put it in the output.  */
-	*out++ = '\n';
+        /* If the newline isn't quoted, put it in the output.  */
+        *out++ = '\n';
 
       /* Now copy the following line to the output.
-	 Stop when we find backslashes followed by a newline.  */
+         Stop when we find backslashes followed by a newline.  */
       while (*in != '\0')
-	if (*in == '\\')
-	  {
-	    p = in + 1;
-	    while (*p == '\\')
-	      ++p;
-	    if (*p == '\n')
-	      {
-		in = p;
-		break;
-	      }
-	    while (in < p)
-	      *out++ = *in++;
-	  }
-	else
-	  *out++ = *in++;
+        if (*in == '\\')
+          {
+            p = in + 1;
+            while (*p == '\\')
+              ++p;
+            if (*p == '\n')
+              {
+                in = p;
+                break;
+              }
+            while (in < p)
+              *out++ = *in++;
+          }
+        else
+          *out++ = *in++;
     }
 
   *out = '\0';
@@ -164,23 +137,14 @@ print_spaces (unsigned int n)
    This string lives in static, re-used memory.  */
 
 const char *
-#if HAVE_ANSI_COMPILER && USE_VARIADIC && HAVE_STDARG_H
 concat (unsigned int num, ...)
-#else
-concat (num, va_alist)
-     unsigned int num;
-     va_dcl
-#endif
 {
   static unsigned int rlen = 0;
   static char *result = NULL;
-  int ri = 0;
-
-#if USE_VARIADIC
+  unsigned int ri = 0;
   va_list args;
-#endif
 
-  VA_START (args, num);
+  va_start (args, num);
 
   while (num-- > 0)
     {
@@ -200,43 +164,118 @@ concat (num, va_alist)
       ri += l;
     }
 
-  VA_END (args);
+  va_end (args);
+
+  /* Get some more memory if we don't have enough space for the
+     terminating '\0'.   */
+  if (ri == rlen)
+    {
+      rlen = (rlen ? rlen : 60) * 2;
+      result = xrealloc (result, rlen);
+    }
 
   result[ri] = '\0';
 
   return result;
 }
 
-/* Print a message on stdout.  */
+
+/* Return a formatted string buffer.
+   LENGTH must be the maximum length of all format arguments, stringified.
+   If we had a standard-compliant vsnprintf() this would be a lot simpler.
+   Maybe in the future we'll include gnulib's version.  */
+
+const char *
+message_s (unsigned int length, int prefix, const char *fmt, ...)
+{
+  static char *buffer = NULL;
+  static unsigned int bsize = 0;
+  char *bp;
+  va_list args;
+
+  /* Compute the maximum buffer size we'll need, and make sure we have it.  */
+  length += strlen (fmt) + strlen (program) + 4 + INTEGER_LENGTH + 2;
+  if (length > bsize)
+    {
+      bsize = length * 2;
+      buffer = xrealloc (buffer, bsize);
+    }
+
+  bp = buffer;
+  if (prefix)
+    {
+      if (makelevel == 0)
+        sprintf (bp, "%s: ", program);
+      else
+        sprintf (bp, "%s[%u]: ", program, makelevel);
+      bp += strlen (buffer);
+    }
+
+  va_start (args, fmt);
+  vsprintf (bp, fmt, args);
+  va_end (args);
+
+  return buffer;
+}
+
+/* Return a formatted error message in a buffer.
+   LENGTH must be the maximum length of all format arguments, stringified.  */
+
+const char *
+error_s (unsigned int length, const gmk_floc *flocp, const char *fmt, ...)
+{
+  static char *buffer = NULL;
+  static unsigned int bsize = 0;
+  char *bp;
+  va_list args;
+
+  /* Compute the maximum buffer size we'll need, and make sure we have it.  */
+  length += (strlen (fmt) + strlen (program) + 4 + INTEGER_LENGTH + 2
+             + (flocp && flocp->filenm ? strlen (flocp->filenm) : 0));
+  if (length > bsize)
+    {
+      bsize = length * 2;
+      buffer = xrealloc (buffer, bsize);
+    }
+
+  bp = buffer;
+  if (flocp && flocp->filenm)
+    sprintf (bp, "%s:%lu: ", flocp->filenm, flocp->lineno);
+  else if (makelevel == 0)
+    sprintf (bp, "%s: ", program);
+  else
+    sprintf (bp, "%s[%u]: ", program, makelevel);
+  bp += strlen (bp);
+
+  va_start (args, fmt);
+  vsprintf (bp, fmt, args);
+  va_end (args);
+
+  return buffer;
+}
+
+/* Print a message on stdout.  We could use message_s() to format it but then
+   we'd need a va_list version...  */
 
 void
-#if HAVE_ANSI_COMPILER && USE_VARIADIC && HAVE_STDARG_H
 message (int prefix, const char *fmt, ...)
-#else
-message (prefix, fmt, va_alist)
-     int prefix;
-     const char *fmt;
-     va_dcl
-#endif
 {
-#if USE_VARIADIC
   va_list args;
-#endif
 
-  log_working_directory (1);
+  log_working_directory (1, 0);
 
   if (fmt != 0)
     {
       if (prefix)
-	{
-	  if (makelevel == 0)
-	    printf ("%s: ", program);
-	  else
-	    printf ("%s[%u]: ", program, makelevel);
-	}
-      VA_START (args, fmt);
-      VA_PRINTF (stdout, fmt, args);
-      VA_END (args);
+        {
+          if (makelevel == 0)
+            printf ("%s: ", program);
+          else
+            printf ("%s[%u]: ", program, makelevel);
+        }
+      va_start (args, fmt);
+      vfprintf (stdout, fmt, args);
+      va_end (args);
       putchar ('\n');
     }
 
@@ -246,20 +285,11 @@ message (prefix, fmt, va_alist)
 /* Print an error message.  */
 
 void
-#if HAVE_ANSI_COMPILER && USE_VARIADIC && HAVE_STDARG_H
-error (const struct floc *flocp, const char *fmt, ...)
-#else
-error (flocp, fmt, va_alist)
-     const struct floc *flocp;
-     const char *fmt;
-     va_dcl
-#endif
+error (const gmk_floc *flocp, const char *fmt, ...)
 {
-#if USE_VARIADIC
   va_list args;
-#endif
 
-  log_working_directory (1);
+  log_working_directory (1, 0);
 
   if (flocp && flocp->filenm)
     fprintf (stderr, "%s:%lu: ", flocp->filenm, flocp->lineno);
@@ -268,9 +298,9 @@ error (flocp, fmt, va_alist)
   else
     fprintf (stderr, "%s[%u]: ", program, makelevel);
 
-  VA_START(args, fmt);
-  VA_PRINTF (stderr, fmt, args);
-  VA_END (args);
+  va_start (args, fmt);
+  vfprintf (stderr, fmt, args);
+  va_end (args);
 
   putc ('\n', stderr);
   fflush (stderr);
@@ -279,20 +309,11 @@ error (flocp, fmt, va_alist)
 /* Print an error message and exit.  */
 
 void
-#if HAVE_ANSI_COMPILER && USE_VARIADIC && HAVE_STDARG_H
-fatal (const struct floc *flocp, const char *fmt, ...)
-#else
-fatal (flocp, fmt, va_alist)
-     const struct floc *flocp;
-     const char *fmt;
-     va_dcl
-#endif
+fatal (const gmk_floc *flocp, const char *fmt, ...)
 {
-#if USE_VARIADIC
   va_list args;
-#endif
 
-  log_working_directory (1);
+  log_working_directory (1, 0);
 
   if (flocp && flocp->filenm)
     fprintf (stderr, "%s:%lu: *** ", flocp->filenm, flocp->lineno);
@@ -301,18 +322,20 @@ fatal (flocp, fmt, va_alist)
   else
     fprintf (stderr, "%s[%u]: *** ", program, makelevel);
 
-  VA_START(args, fmt);
-  VA_PRINTF (stderr, fmt, args);
-  VA_END (args);
+  va_start (args, fmt);
+  vfprintf (stderr, fmt, args);
+  va_end (args);
 
   fputs (_(".  Stop.\n"), stderr);
 
+  log_working_directory (0, 1);
+
   die (2);
 }
 
 #ifndef HAVE_STRERROR
 
-#undef	strerror
+#undef  strerror
 
 char *
 strerror (int errnum)
@@ -476,7 +499,7 @@ end_of_token_w32 (const char *s, char stopchar)
   int backslash = 0;
 
   while (*p != '\0' && *p != stopchar
-	 && (backslash || !isblank ((unsigned char)*p)))
+         && (backslash || !isblank ((unsigned char)*p)))
     {
       if (*p++ == '\\')
         {
@@ -525,7 +548,7 @@ find_next_token (const char **ptr, unsigned int *lengthptr)
 }
 
 
-/* Copy a chain of `struct dep'.  For 2nd expansion deps, dup the name.  */
+/* Copy a chain of 'struct dep'.  For 2nd expansion deps, dup the name.  */
 
 struct dep *
 copy_dep_chain (const struct dep *d)
@@ -543,9 +566,9 @@ copy_dep_chain (const struct dep *d)
 
       c->next = 0;
       if (firstnew == 0)
-	firstnew = lastnew = c;
+        firstnew = lastnew = c;
       else
-	lastnew = lastnew->next = c;
+        lastnew = lastnew->next = c;
 
       d = d->next;
     }
@@ -606,8 +629,36 @@ strcasecmp (const char *s1, const char *s2)
     }
 }
 #endif
+
+#if !HAVE_STRNCASECMP && !HAVE_STRNICMP && !HAVE_STRNCMPI
+
+/* If we don't have strncasecmp() (from POSIX), or anything that can
+   substitute for it, define our own version.  */
+
+int
+strncasecmp (const char *s1, const char *s2, int n)
+{
+  while (n-- > 0)
+    {
+      int c1 = (int) *(s1++);
+      int c2 = (int) *(s2++);
+
+      if (isalpha (c1))
+        c1 = tolower (c1);
+      if (isalpha (c2))
+        c2 = tolower (c2);
+
+      if (c1 != '\0' && c1 == c2)
+        continue;
+
+      return (c1 - c2);
+    }
+
+  return 0;
+}
+#endif
 
-#ifdef	GETLOADAVG_PRIVILEGED
+#ifdef  GETLOADAVG_PRIVILEGED
 
 #ifdef POSIX
 
@@ -620,7 +671,7 @@ strcasecmp (const char *s1, const char *s2)
 #undef HAVE_SETREUID
 #undef HAVE_SETREGID
 
-#else	/* Not POSIX.  */
+#else   /* Not POSIX.  */
 
 /* Some POSIX.1 systems have the seteuid and setegid functions.  In a
    POSIX-like system, they are the best thing to use.  However, some
@@ -630,30 +681,30 @@ strcasecmp (const char *s1, const char *s2)
 #undef HAVE_SETEUID
 #undef HAVE_SETEGID
 
-#endif	/* POSIX.  */
+#endif  /* POSIX.  */
 
-#ifndef	HAVE_UNISTD_H
+#ifndef HAVE_UNISTD_H
 extern int getuid (), getgid (), geteuid (), getegid ();
 extern int setuid (), setgid ();
 #ifdef HAVE_SETEUID
 extern int seteuid ();
 #else
-#ifdef	HAVE_SETREUID
+#ifdef  HAVE_SETREUID
 extern int setreuid ();
-#endif	/* Have setreuid.  */
-#endif	/* Have seteuid.  */
+#endif  /* Have setreuid.  */
+#endif  /* Have seteuid.  */
 #ifdef HAVE_SETEGID
 extern int setegid ();
 #else
-#ifdef	HAVE_SETREGID
+#ifdef  HAVE_SETREGID
 extern int setregid ();
-#endif	/* Have setregid.  */
-#endif	/* Have setegid.  */
-#endif	/* No <unistd.h>.  */
+#endif  /* Have setregid.  */
+#endif  /* Have setegid.  */
+#endif  /* No <unistd.h>.  */
 
 /* Keep track of the user and group IDs for user- and make- access.  */
 static int user_uid = -1, user_gid = -1, make_uid = -1, make_gid = -1;
-#define	access_inited	(user_uid != -1)
+#define access_inited   (user_uid != -1)
 static enum { make, user } current_access;
 
 
@@ -670,7 +721,7 @@ log_access (const char *flavor)
      run in a child fork whose stdout is piped.  */
 
   fprintf (stderr, _("%s: user %lu (real %lu), group %lu (real %lu)\n"),
-	   flavor, (unsigned long) geteuid (), (unsigned long) getuid (),
+           flavor, (unsigned long) geteuid (), (unsigned long) getuid (),
            (unsigned long) getegid (), (unsigned long) getgid ());
   fflush (stderr);
 }
@@ -696,14 +747,14 @@ init_access (void)
 #endif
 }
 
-#endif	/* GETLOADAVG_PRIVILEGED */
+#endif  /* GETLOADAVG_PRIVILEGED */
 
 /* Give the process appropriate permissions for access to
    user data (i.e., to stat files, or to spawn a child process).  */
 void
 user_access (void)
 {
-#ifdef	GETLOADAVG_PRIVILEGED
+#ifdef  GETLOADAVG_PRIVILEGED
 
   if (!access_inited)
     init_access ();
@@ -716,7 +767,7 @@ user_access (void)
      We now want to set the effective user and group IDs to the real IDs,
      which are the IDs of the process that exec'd make.  */
 
-#ifdef	HAVE_SETEUID
+#ifdef  HAVE_SETEUID
 
   /* Modern systems have the seteuid/setegid calls which set only the
      effective IDs, which is ideal.  */
@@ -724,9 +775,9 @@ user_access (void)
   if (seteuid (user_uid) < 0)
     pfatal_with_name ("user_access: seteuid");
 
-#else	/* Not HAVE_SETEUID.  */
+#else   /* Not HAVE_SETEUID.  */
 
-#ifndef	HAVE_SETREUID
+#ifndef HAVE_SETREUID
 
   /* System V has only the setuid/setgid calls to set user/group IDs.
      There is an effective ID, which can be set by setuid/setgid.
@@ -739,7 +790,7 @@ user_access (void)
   if (setuid (user_uid) < 0)
     pfatal_with_name ("user_access: setuid");
 
-#else	/* HAVE_SETREUID.  */
+#else   /* HAVE_SETREUID.  */
 
   /* In 4BSD, the setreuid/setregid calls set both the real and effective IDs.
      They may be set to themselves or each other.  So you have two alternatives
@@ -751,14 +802,14 @@ user_access (void)
   if (setreuid (make_uid, user_uid) < 0)
     pfatal_with_name ("user_access: setreuid");
 
-#endif	/* Not HAVE_SETREUID.  */
-#endif	/* HAVE_SETEUID.  */
+#endif  /* Not HAVE_SETREUID.  */
+#endif  /* HAVE_SETEUID.  */
 
-#ifdef	HAVE_SETEGID
+#ifdef  HAVE_SETEGID
   if (setegid (user_gid) < 0)
     pfatal_with_name ("user_access: setegid");
 #else
-#ifndef	HAVE_SETREGID
+#ifndef HAVE_SETREGID
   if (setgid (user_gid) < 0)
     pfatal_with_name ("user_access: setgid");
 #else
@@ -771,7 +822,7 @@ user_access (void)
 
   log_access (_("User access"));
 
-#endif	/* GETLOADAVG_PRIVILEGED */
+#endif  /* GETLOADAVG_PRIVILEGED */
 }
 
 /* Give the process appropriate permissions for access to
@@ -779,7 +830,7 @@ user_access (void)
 void
 make_access (void)
 {
-#ifdef	GETLOADAVG_PRIVILEGED
+#ifdef  GETLOADAVG_PRIVILEGED
 
   if (!access_inited)
     init_access ();
@@ -789,11 +840,11 @@ make_access (void)
 
   /* See comments in user_access, above.  */
 
-#ifdef	HAVE_SETEUID
+#ifdef  HAVE_SETEUID
   if (seteuid (make_uid) < 0)
     pfatal_with_name ("make_access: seteuid");
 #else
-#ifndef	HAVE_SETREUID
+#ifndef HAVE_SETREUID
   if (setuid (make_uid) < 0)
     pfatal_with_name ("make_access: setuid");
 #else
@@ -802,11 +853,11 @@ make_access (void)
 #endif
 #endif
 
-#ifdef	HAVE_SETEGID
+#ifdef  HAVE_SETEGID
   if (setegid (make_gid) < 0)
     pfatal_with_name ("make_access: setegid");
 #else
-#ifndef	HAVE_SETREGID
+#ifndef HAVE_SETREGID
   if (setgid (make_gid) < 0)
     pfatal_with_name ("make_access: setgid");
 #else
@@ -819,7 +870,7 @@ make_access (void)
 
   log_access (_("Make access"));
 
-#endif	/* GETLOADAVG_PRIVILEGED */
+#endif  /* GETLOADAVG_PRIVILEGED */
 }
 
 /* Give the process appropriate permissions for a child process.
@@ -827,7 +878,7 @@ make_access (void)
 void
 child_access (void)
 {
-#ifdef	GETLOADAVG_PRIVILEGED
+#ifdef  GETLOADAVG_PRIVILEGED
 
   if (!access_inited)
     abort ();
@@ -835,7 +886,7 @@ child_access (void)
   /* Set both the real and effective UID and GID to the user's.
      They cannot be changed back to make's.  */
 
-#ifndef	HAVE_SETREUID
+#ifndef HAVE_SETREUID
   if (setuid (user_uid) < 0)
     pfatal_with_name ("child_access: setuid");
 #else
@@ -843,7 +894,7 @@ child_access (void)
     pfatal_with_name ("child_access: setreuid");
 #endif
 
-#ifndef	HAVE_SETREGID
+#ifndef HAVE_SETREGID
   if (setgid (user_gid) < 0)
     pfatal_with_name ("child_access: setgid");
 #else
@@ -853,7 +904,7 @@ child_access (void)
 
   log_access (_("Child access"));
 
-#endif	/* GETLOADAVG_PRIVILEGED */
+#endif  /* GETLOADAVG_PRIVILEGED */
 }
 
 #ifdef NEED_GET_PATH_MAX
@@ -866,9 +917,9 @@ get_path_max (void)
     {
       long int x = pathconf ("/", _PC_PATH_MAX);
       if (x > 0)
-	value = x;
+        value = x;
       else
-	return MAXPATHLEN;
+        return MAXPATHLEN;
     }
 
   return value;
@@ -876,6 +927,85 @@ get_path_max (void)
 #endif
 
 
+/* Provide support for temporary files.  */
+
+#ifndef HAVE_STDLIB_H
+# ifdef HAVE_MKSTEMP
+int mkstemp (char *template);
+# else
+char *mktemp (char *template);
+# endif
+#endif
+
+/* This is only used by output-sync, and it may not be portable.  */
+#ifdef OUTPUT_SYNC
+
+/* Returns a file descriptor to a temporary file.  The file is automatically
+   closed/deleted on exit.  Don't use a FILE* stream.  */
+int
+open_tmpfd ()
+{
+  int fd = -1;
+  FILE *tfile = tmpfile ();
+
+  if (! tfile)
+    pfatal_with_name ("tmpfile");
+
+  /* Create a duplicate so we can close the stream.  */
+  fd = dup (fileno (tfile));
+  if (fd < 0)
+    pfatal_with_name ("dup");
+
+  fclose (tfile);
+
+  return fd;
+}
+
+#endif
+
+FILE *
+open_tmpfile (char **name, const char *template)
+{
+#ifdef HAVE_FDOPEN
+  int fd;
+#endif
+
+#if defined HAVE_MKSTEMP || defined HAVE_MKTEMP
+# define TEMPLATE_LEN   strlen (template)
+#else
+# define TEMPLATE_LEN   L_tmpnam
+#endif
+  *name = xmalloc (TEMPLATE_LEN + 1);
+  strcpy (*name, template);
+
+#if defined HAVE_MKSTEMP && defined HAVE_FDOPEN
+  /* It's safest to use mkstemp(), if we can.  */
+  fd = mkstemp (*name);
+  if (fd == -1)
+    return 0;
+  return fdopen (fd, "w");
+#else
+# ifdef HAVE_MKTEMP
+  (void) mktemp (*name);
+# else
+  (void) tmpnam (*name);
+# endif
+
+# ifdef HAVE_FDOPEN
+  /* Can't use mkstemp(), but guard against a race condition.  */
+  fd = open (*name, O_CREAT|O_EXCL|O_WRONLY, 0600);
+  if (fd == -1)
+    return 0;
+  return fdopen (fd, "w");
+# else
+  /* Not secure, but what can we do?  */
+  return fopen (*name, "w");
+# endif
+#endif
+}
+
+
+
 /* This code is stolen from gnulib.
    If/when we abandon the requirement to work with K&R compilers, we can
    remove this (and perhaps other parts of GNU make!) and migrate to using
@@ -903,7 +1033,7 @@ get_path_max (void)
    the failure.  That's what the ferror test is checking below.
 
    It's important to detect such failures and exit nonzero because many
-   tools (most notably `make' and other build-management systems) depend
+   tools (most notably 'make' and other build-management systems) depend
    on being able to detect failure in other tools via their exit status.  */
 
 void
